/**************************************************************************/
/* 
   Use kFRACT to set the number of segments.  On a machine with 32Kb RAM 
   it is possible that only 1 segment is required unless many maps
   are to be opened simultaneously.  Increasing the number of segments
   seriously degrades performance                                         */

/**************************************************************************/

#define kFRACT  1

#include "gis.h"
#include "segment.h"

char* Create_segment_file (int);
int Init_segment_file (char*, SEGMENT*);


/**************************************************************************/
/* Public functions                                                       */
/**************************************************************************/

Segment_infile (map_name, mapset, map_fd, seg_name, seg_fd, seg)
char* map_name;
char* mapset;
int* map_fd;
char* seg_name;
int* seg_fd;
SEGMENT* seg;

{
    int nrows, ncols, row;
    CELL* cell;

    strcpy (mapset, G_find_cell (map_name, ""));
    if (mapset == NULL)
	G_fatal_error ("Can't find displayed data layer ");
    if ((*map_fd = G_open_cell_old (map_name, mapset)) < 0)
	G_fatal_error ("Can't open displayed data layer ");

    nrows = G_window_rows ();
    ncols = G_window_cols ();
    cell = G_allocate_cell_buf ();

    strcpy (seg_name, Create_segment_file (nrows));
    *seg_fd = open (seg_name, 2);

    if (*seg_fd < 0)
	G_fatal_error ("Can't open segment disk file ");
    segment_init (seg, *seg_fd, 4);

    for (row = 0; row < nrows; row++)
    {
	if (G_get_map_row (*map_fd, cell, row) < 0)
	    G_fatal_error ("Unable to read data layer into segment file ");

	segment_put_row (seg, cell, row);
    }
}


/**************************************************************************/

Segment_outfile (map_name, mapset, map_fd,
		    seg_name, seg_fd, seg, prompt)
char* map_name;
char* mapset;
int* map_fd;
char* seg_name;
int* seg_fd;
SEGMENT* seg;
char* prompt;

{
    int nrows, row;
    char* mapset_address;

    mapset_address = G_ask_cell_new (prompt, map_name);
    if (mapset_address == NULL)
	G_fatal_error ("Can't continue without filename ");
    strcpy (mapset, mapset_address);
    if ((*map_fd = G_open_cell_new (map_name)) < 0)
	G_fatal_error ("Can't create output layer ");

    nrows = G_window_rows ();

    strcpy (seg_name, Create_segment_file (nrows));
    *seg_fd = open (seg_name, 2);
    if (*seg_fd < 0)
	G_fatal_error ("Can't open segment disk file ");
    segment_init (seg, *seg_fd, 4);
}


/**************************************************************************/

Segment_named_outfile (map_name, mapset, map_fd,
			  seg_name, seg_fd, seg, overwrite, terse)
char* map_name;
char* mapset;
int* map_fd;
char* seg_name;
int* seg_fd;
SEGMENT* seg;
int overwrite;
int terse;

{
    int nrows, row;
    char* mapset_address;
    char prompt [100];
    char message [64];

    mapset_address = G_find_cell (map_name, G_mapset ());
    strcpy (mapset, G_mapset ());
    if (mapset_address != NULL)
    {
      if (!overwrite)
	{
	  sprintf (message, "Raster map '%s' exists ", map_name);
	  G_fatal_error (message);
	}
      else
	{
	  if (!terse)
	    fprintf (stderr, "\nOverwriting raster map '%s' ", map_name);
	}
    }
    else
      {
	if (!terse)
	  fprintf (stderr, "\nCreating raster map '%s' ", map_name);
      }

    if ((*map_fd = G_open_cell_new (map_name)) < 0)
	G_fatal_error ("Can't create output layer ");

    nrows = G_window_rows ();

    strcpy (seg_name, Create_segment_file (nrows));
    *seg_fd = open (seg_name, 2);
    if (*seg_fd < 0)
	G_fatal_error ("Can't open segment disk file ");
    segment_init (seg, *seg_fd, 4);
}


/**************************************************************************/
/* Private functions                                                      */
/**************************************************************************/

char* Create_segment_file (nrows)
int nrows;
{
    int seg_fd;
    char* seg_name;
    int ncols;
    int submatrix_rows, submatrix_cols, length_data_item;

    ncols = G_window_cols ();
    length_data_item = sizeof (CELL);
    submatrix_rows = nrows/kFRACT + 1;
    submatrix_cols = ncols/kFRACT + 1;
    seg_name = G_tempfile ();
    seg_fd = creat (seg_name, 0666);
    segment_format (seg_fd, nrows, ncols,
		    submatrix_rows, submatrix_cols, length_data_item);
    close (seg_fd);
    return seg_name;
}


/**************************************************************************/

int Init_segment_file (seg_name, seg)
char* seg_name;
SEGMENT* seg;

{
    int seg_fd;

    seg_fd = open (seg_name, 2);
    if (seg_fd < 0)
	G_fatal_error ("Can't open segment disk file");
    segment_init (seg, seg_fd, 4);
    return seg_fd;
}
